#include "Well.hpp"

#include <algorithm>

namespace
{
constexpr double gUndefinedValue(std::numeric_limits<double>::quiet_NaN());
}

TrajectoryPoint::TrajectoryPoint(double measureDepth, double inclination, double azimuth)
    : mMeasureDepth(measureDepth)
    , mInclination(inclination)
    , mAzimuth(azimuth)
{
}

bool Well::isEmpty() const
{
    return mPoints.empty();
}

std::size_t Well::getSize() const
{
    return mPoints.size();
}

void Well::addPoint(const TrajectoryPoint & point)
{
    mPoints.push_back(point);
}
TrajectoryPoint & Well::getPoint(std::size_t ind)
{
    return mPoints[ind];
}

const TrajectoryPoint & Well::getPoint(std::size_t ind) const
{
    return mPoints[ind];
}

double Well::getMinTvd() const
{
    auto found(std::min_element(mPoints.cbegin(), mPoints.cend(), [](const TrajectoryPoint & a, const TrajectoryPoint & b)
    {
        return a.mTrueVerticalDepth < b.mTrueVerticalDepth;
    }));

    if (found == mPoints.cend())
        return gUndefinedValue;

    return found->mTrueVerticalDepth;
}

double Well::getMaxTvd() const
{
    auto found(std::max_element(mPoints.cbegin(), mPoints.cend(), [](const TrajectoryPoint & a, const TrajectoryPoint & b)
    {
        return a.mTrueVerticalDepth < b.mTrueVerticalDepth;
    }));

    if (found == mPoints.cend())
        return gUndefinedValue;

    return found->mTrueVerticalDepth;
}

double Well::getMinVs() const
{
    auto found(std::min_element(mPoints.cbegin(), mPoints.cend(), [](const TrajectoryPoint & a, const TrajectoryPoint & b)
    {
        return a.mAzimuthProjection < b.mAzimuthProjection;
    }));

    if (found == mPoints.cend())
        return gUndefinedValue;

    return found->mAzimuthProjection;
}

double Well::getMaxVs() const
{
    auto found(std::max_element(mPoints.cbegin(), mPoints.cend(), [](const TrajectoryPoint & a, const TrajectoryPoint & b)
    {
        return a.mAzimuthProjection < b.mAzimuthProjection;
    }));

    if (found == mPoints.cend())
        return gUndefinedValue;

    return found->mAzimuthProjection;
}


