#pragma once

#include <vector>
#include <QVector3D>

/// Точка траектории
struct TrajectoryPoint final
{
    TrajectoryPoint(double measureDepth, double inclination, double azimuth);

    /// Исходные данные
    double mMeasureDepth;
    double mInclination;
    double mAzimuth;

    /// Вычисленные данные
    /// mX и mY - координаты на горизонтальной плоскости
    /// mTrueVerticalDepth - глубина скважины
    /// mAzimuthProjection - проекция на вертикальную плоскость, повёрнутую на величину азимута
    double mX = 0.0;
    double mY = 0.0;
    double mTrueVerticalDepth = 0.0;
    double mAzimuthProjection = 0.0;

    void upByNormal(TrajectoryPoint prev, TrajectoryPoint next, double val){
        QVector3D first(prev.mX, prev.mY, prev.mTrueVerticalDepth);
        QVector3D second(next.mX, next.mY, next.mTrueVerticalDepth);
        QVector3D direction(1.0 , 0.0, 0.0);
        QVector3D delta = (second - first).normalized();

        QVector3D normal = QVector3D::normal(direction, delta) * val + QVector3D(this->mX, this->mY, this->mTrueVerticalDepth);

        this->mX = normal[0];
        this->mY = normal[1];
        this->mTrueVerticalDepth = normal[2];
        //double x3 = (x2 - x1) * sq;
        //this->mTrueVerticalDepth -= val;
    }

};

/// Траектория скважины
class Well final
{
public:
    bool isEmpty() const;
    std::size_t getSize() const;

    void addPoint(const TrajectoryPoint & point);
    TrajectoryPoint & getPoint(std::size_t ind);
    const TrajectoryPoint & getPoint(std::size_t ind) const;
    double getMinTvd() const;
    double getMaxTvd() const;

    double getMinVs() const;
    double getMaxVs() const;
    std::vector<TrajectoryPoint> mPoints;
};

