#include "PlotterWidget.hpp"
#include "utils.hpp"
#include "Objects/Well.hpp"



namespace
{
const QColor gBackgroundColor(230, 230, 230);
const QColor gGridColor(30, 30, 30);
}

PlotterWidget::PlotterWidget(QWidget * parent)
    : QCustomPlot(parent)
{
    yAxis->setRangeReversed(true);
    setInteractions(QCP::iRangeDrag | QCP::iRangeZoom);

    xAxis->setLabel("VS");
    yAxis->setLabel("TVD");

    setBackground(gBackgroundColor);

    xAxis->setBasePen(gGridColor);
    xAxis->setSubTickPen(gGridColor);
    xAxis->setTickPen(gGridColor);
    xAxis->setTickLabelColor(gGridColor);
    xAxis->setLabelColor(gGridColor);

    yAxis->setBasePen(gGridColor);
    yAxis->setSubTickPen(gGridColor);
    yAxis->setTickPen(gGridColor);
    yAxis->setTickLabelColor(gGridColor);
    yAxis->setLabelColor(gGridColor);

    mLogCurve = new QCPCurve(xAxis, yAxis);
    mLogCurve->setPen(QPen(Qt::blue, 1.25));
    mLogCurve->setBrush(QBrush(QColor(0, 0, 255, 20)));

    mTrajectoryCurve = new QCPCurve(xAxis, yAxis);
    mTrajectoryCurve->setPen(QPen(Qt::red, 2));


}

void PlotterWidget::setAzimuthVS(double azimuthRad)
{
    mAzimuthRad = azimuthRad;
}

void PlotterWidget::drawTrajectory(const Well & well)
{
    mTrajectoryCurve->data()->clear();

    std::size_t pointsCount(well.getSize());
    for(std::size_t i(0); i < pointsCount; ++i)
    {
        const TrajectoryPoint & currentPoint(well.getPoint(i));
        mTrajectoryCurve->addData(currentPoint.mAzimuthProjection, currentPoint.mTrueVerticalDepth);
    }

    replot();
}

void PlotterWidget::drawCurve(const Well & well)
{

    mLogCurve->data()->clear();

    std::size_t pointsCount(well.getSize());
    for(std::size_t i(0); i < pointsCount; ++i)
    {
        const TrajectoryPoint & currentPoint(well.getPoint(i));
        mLogCurve->addData(currentPoint.mAzimuthProjection, currentPoint.mTrueVerticalDepth);
    }

    replot();
}

void PlotterWidget::scaleByTrajectory(const Well & well)
{
    xAxis->setRange(well.getMinVs(), well.getMaxVs());
    yAxis->setRange(well.getMinTvd(), well.getMaxTvd());

    replot();
}

